@@ -15,12 +15,12 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dbi_postgresql.ml,v 1.4 2004/08/04 17:14:14 rwmj Exp $
+ * $Id: dbi_postgresql.ml,v 1.11 2005/01/25 19:07:12 ChriS Exp $
  *)
 
 open Printf
 
-module P = Postgresql
+module Pg = Postgresql
 
 (* PCRE regular expressions for parsing timestamps and intervals. *)
 let re_timestamp =
@@ -72,8 +72,22 @@ let timestamp_of_string =
            else Some (let tz = int_of_string sub.(9) in
                       if sub.(8) = "-" then -tz else tz);
        })
-    with
-      Not_found -> failwith ("timestamp_of_string: bad timestamp: " ^ str)
+    with Not_found ->
+      (* FIXME: improve this *)
+      if str = "infinity" then
+	({ Dbi.year = max_int;
+	   Dbi.month = 12;
+           Dbi.day = 31; },
+	 { Dbi.hour = 23;  Dbi.min = 59;  Dbi.sec = 59;
+           Dbi.microsec = 1000000;  Dbi.timezone = None })
+      else if str = "infinity" then
+	({ Dbi.year = min_int;
+	   Dbi.month = 1;
+           Dbi.day = 1; },
+	 { Dbi.hour = 0;  Dbi.min = 0;  Dbi.sec = 0;
+           Dbi.microsec = 0;  Dbi.timezone = None })
+      else
+	failwith ("timestamp_of_string: bad timestamp: " ^ str)
 
 let string_of_interval (date, time) =
   sprintf "'%d years %d mons %d days %02d:%02d:%02d.%d'"
@@ -110,50 +124,60 @@ let time_of_string s =
    in a SQL query. *)
 let encode_sql_t = function
   | `Null -> "NULL"
-  | `Int i -> string_of_int i
+  | `Int i ->
+      (* As we use [`Int] for INTEGER and SMALLINT, we quote the
+	 integer to delay type resolution and so allow the system to
+	 use an index (postgres manual "8.1.1. Integer Types"). *)
+      "'" ^ string_of_int i ^ "'"
   | `Float f -> string_of_float f
   | `String s -> Dbi.string_escaped s
-  | `Bool b -> if b then "'t'" else "'f'"
-      (*  | `Bigint i -> string_of_big_int i *)
+  | `Bool b -> if b then "TRUE" else "FALSE"
+  | `Bigint i -> Big_int.string_of_big_int i
   | `Decimal d -> Dbi.Decimal.to_string d
   | `Date d -> sprintf "'%04i-%02i-%02i'" d.Dbi.year d.Dbi.month d.Dbi.day
   | `Time t -> sprintf "'%02i:%02i:%02i'" t.Dbi.hour t.Dbi.min t.Dbi.sec
   | `Timestamp t -> string_of_timestamp t
   | `Interval i -> string_of_interval i
   | `Blob s -> Dbi.string_escaped s (* FIXME *)
-  | `Binary s -> "'" ^ P.escape_bytea s ^ "'::bytea"
+  | `Binary s -> "'" ^ Pg.escape_bytea s ^ "'::bytea"
   | `Unknown s -> s
 
+
+
+(* Markus now has symbolic names for the types, so we do not need to
+   make the numeric conversions ourselves.  (Hard-coded OIDs from
+   pg_type.h tell the type.) *)
 let decode_sql_t is_null ty v =
   if is_null then `Null
   else begin
-    (* Hard-coded OIDs from pg_type.h tell the type *)
     match ty with
-    | 16 -> `Bool(v = "t")
-    | 17 (* bytea *) -> `Binary (P.unescape_bytea v)
-    | 18   (* char *) -> `String v
-(*    | 20 (* int8 *) -> `Bigint(Big_int.big_int_of_string v) *)
-    | 20   (* FIXXXME *) -> `Int (int_of_string v)
-    | 21   (* int2 *)
-    | 23   (* int4 *) -> `Int (int_of_string v)
-    | 25   (* text *) -> `String v
-    | 700  (* float4 *)
-    | 701  (* float8 *) -> `Float (float_of_string v)
-    | 1043 (* varchar *) -> `String v
-    | 1082 (* date *) -> `Date (date_of_string v)
-    | 1083 (* time *)
-    | 1266 (* timetz *) -> `Time (time_of_string v)
-    | 1114 (* timestamp *)
-    | 1184 (* timestamptz *) -> `Timestamp (timestamp_of_string v)
-    | 1186 (* interval *) -> `Interval (interval_of_string v)
-    | 1700 (* numeric *) -> `Decimal(Dbi.Decimal.of_string v)
-    | _ -> `Unknown v
+    | Pg.INT2
+    | Pg.INT4    -> `Int (int_of_string v)
+    | Pg.FLOAT4
+    | Pg.FLOAT8  -> `Float (float_of_string v)
+    | Pg.TEXT
+    | Pg.BPCHAR
+    | Pg.CHAR
+    | Pg.VARCHAR -> `String v
+    | Pg.BOOL    -> `Bool(v = "t")
+    | Pg.INT8    -> `Bigint(Big_int.big_int_of_string v)
+    | Pg.NUMERIC -> `Decimal(Dbi.Decimal.of_string v)
+    | Pg.DATE    -> `Date (date_of_string v)
+    | Pg.ABSTIME
+    | Pg.RELTIME
+    | Pg.TIME
+    | Pg.TIMETZ -> `Time (time_of_string v) (* FIXME: what about TZ? *)
+    | Pg.TIMESTAMP
+    | Pg.TIMESTAMPTZ -> `Timestamp (timestamp_of_string v)
+    | Pg.TINTERVAL
+    | Pg.INTERVAL -> `Interval (interval_of_string v)
+    | Pg.BYTEA   -> `Binary (Pg.unescape_bytea v)
+    | _ ->
+	`Unknown v
   end
 
-(* XXX Markus has hidden the oid type. *)
-let int_of_oid oid : int = Obj.magic oid
 
-class statement dbh (conn : P.connection) in_transaction original_query =
+class statement dbh (conn : Pg.connection) in_transaction original_query =
   let query = Dbi.split_query original_query in
 object (self)
   inherit Dbi.statement dbh
@@ -190,24 +214,24 @@ object (self)
     let res = conn#exec query in
 
     match res#status with
-	P.Empty_query ->
+	Pg.Empty_query ->
 	  ()
-      | P.Command_ok ->
+      | Pg.Command_ok ->
 	  ()
-      | P.Tuples_ok ->
+      | Pg.Tuples_ok ->
 	  tuples <- Some res;
 	  name_list <- None;
 	  next_tuple <- 0;
 	  ntuples <- res#ntuples;
 	  nfields <- res#nfields
-      | P.Copy_out
-      | P.Copy_in ->
+      | Pg.Copy_out
+      | Pg.Copy_in ->
 	  failwith "XXX copyin/copyout not implemented"
-      | P.Bad_response
-      | P.Fatal_error ->
+      | Pg.Bad_response
+      | Pg.Fatal_error ->
 	  (* dbh#close (); -- used to do this, not a good idea *)
 	  raise (Dbi.SQL_error (res#error))
-      | P.Nonfatal_error ->
+      | Pg.Nonfatal_error ->
 	  prerr_endline ("Dbi_postgres: non-fatal error: " ^ res#error)
 
   method fetch1 () =
@@ -227,7 +251,7 @@ object (self)
 	    if i < nfields then (
 	      let field = (* FIXME: what about binary tuples?? *)
 		decode_sql_t (tuples#getisnull next_tuple i)
-		  (int_of_oid (tuples#ftype i))
+		  (tuples#ftype i)
 		  (tuples#getvalue next_tuple i) in
 	      field :: loop (i+1)
 	    ) else
@@ -285,7 +309,7 @@ and connection ?host ?port ?user ?password database =
    * to pass arbitrary parameters to the underlying database.
    *)
   let conn =
-    new P.connection ?host ?port ?user ?password ~dbname:database () in
+    new Pg.connection ?host ?port ?user ?password ~dbname:database () in
 
   (* We pass this reference around to the statement class so that all
    * statements belonging to this connection can keep track of our
@@ -334,7 +358,7 @@ object (self)
     super#close ()
 
   initializer
-    if conn#status = P.Bad then
+    if conn#status = Pg.Bad then
       raise (Dbi.SQL_error conn#error_message)
 end
 
